//---------------------------------------------------------------------------
// LNDMSVC.C
//---------------------------------------------------------------------------
//    
// Sample of how to call the LINDO DLL using the
// 16-bit version Microsoft Visual C compiler.
//
// This application sends LINDO the following simple LP
// model:
//
//    MAX 20X + 30Y
//    ST
//       X < 50
//       Y < 60
//       X + 2Y < 120
//    END
//
// The model is then solved and the optimal variable values
// are reported in a dialog box.
//
//---------------------------------------------------------------------------
//
// "Make" file:
//
//    CFLAGS = /nologo /YX /G2 /Zp1 /W3 /AS /Od /D "_DEBUG" /FR /GA /Zi
//    LFLAGS = /NOD 
//    LIBS   = oldnames libw commdlg shell olecli olesvr slibcew LIBW 
//    
//    LNDMSVC.EXE : LNDMSVC.OBJ LNDMSVC.DEF LNDMSVC.MAK
//         link $(LFLAGS) LNDMSVC,,, $(LIBS), LNDMSVC
//    
//    LNDMSVC.OBJ: LNDMSVC.C LNDMSVC.MAK
//         cl $(CFLAGS) /c LNDMSVC.C
//
//---------------------------------------------------------------------------
//
// "DEF" file:
//    
//    NAME            LNDMSVC
//    
//    DESCRIPTION     'Sample call to LINDO DLL using MSVC'
//    EXETYPE         WINDOWS
//    STUB            'WINSTUB.EXE'
//    CODE            PRELOAD MOVEABLE DISCARDABLE
//    DATA            PRELOAD MOVEABLE MULTIPLE
//    HEAPSIZE        1024
//    STACKSIZE       8192
//    
//    
//---------------------------------------------------------------------------

#include "windows.h"    // required by all Windows applications
#include "stdio.h"      // we use the sprintf function
#include "lnddllc.h"    // defines ordinal indices of LINDO entry points

//---------------------------------------------------------------------------

// Function definitions used to call into LINDO library

typedef long (FAR PASCAL *FPROC)();
typedef long (FAR PASCAL *FPROC0)( int);
typedef long (FAR PASCAL *FPROC1)( void far *, int);
typedef long (FAR PASCAL *FPROC2)( void far *, void far *, int);
typedef long (FAR PASCAL *FPROC3)( void far *, void far *, void far *, int);
typedef long (FAR PASCAL *FPROC4)( void far *, void far *, void far *, 
 void far *, int);
typedef long (FAR PASCAL *FPROC5)( void far *, void far *, void far *, 
 void far *, void far *, int);
typedef long (FAR PASCAL *FPROC7)( void far *, void far *, void far *, 
 void far *, void far *, void far *, void far *, int);

//---------------------------------------------------------------------------

// Main Program

int PASCAL WinMain(HANDLE hInstance, HANDLE hPrevInstance, 
 LPSTR lpCmdLine, int nCmdShow)
{

// Locals

   char cBuff[256];
   char *cTitle = "LNDMSVC";
   FPROC fp;
   HANDLE hDLL;
   long lTRUBLE;

// Load LINDO DLL

   hDLL = LoadLibrary( "LINDO.DLL");
   if ( hDLL < 32)
   {
      sprintf( cBuff, "Failed to load LINDO library, Error Code: %d", hDLL);
      MessageBox( NULL, cBuff, cTitle, MB_OK);
      return( FALSE);
   }

// Get address of main entry point into LINDO DLL

   fp = (FPROC) GetProcAddress( hDLL, "Win386LibEntry");

// Call ILINDO and INIT to do LINDO initialization

   (*(FPROC0) fp)( ORD_ILINDO);
   (*(FPROC0) fp)( ORD_INIT);

// Capture standard output from LINDO in a file

   {
      long lUnit=60, lFname=9, lZero = 0;
      char *cFname = "LINDO.OUT";

// Use LUNOPN to open a file

      (*(FPROC7) fp)( &lUnit, &lFname, cFname, &lZero, &lZero, &lZero, 
       &lZero, ORD_LUNOPN);

// Tell LINDO to send all standard output to this file

      lUnit = 60;
      (*(FPROC1) fp)( &lUnit, ORD_CAPOUT);
   }

// Put LINDO in TERSE mode

   {
      long lTerse = 0;
      (*(FPROC1) fp)( &lTerse, ORD_QUIET);
   }

// Call DEFROW to define the rows

   {
      int i;
      long lIDIR = -1, lIDROW;
      float fRHS = 0, fConstraintRHS[3]={50,60,120};

// Define objective row

      (*(FPROC4)fp)( &lIDIR, &fRHS, &lIDROW, 
       &lTRUBLE, ORD_DEFROW);

// Define constraint rows

      lIDIR = 1;
      for ( i = 2; i <= 4; i++)
      {
         (*(FPROC4)fp)( &lIDIR, &fConstraintRHS[ i - 2], &lIDROW, 
          &lTRUBLE, ORD_DEFROW);
      }
   } 

// Call APPCOL to define our two columns

   {   
      char *cNameX = "X       ";   
      char *cNameY = "Y       ";   
      long lNonz = 3;
      float fValX[] = { 20, 1, 1};
      float fValY[] = { 30, 1, 2};
      long lIroX[] = { 1, 2, 4};
      long lIroY[] = { 1, 3, 4};

// Send LINDO column X ...

      (*(FPROC5)fp)( cNameX, &lNonz, fValX, 
       lIroX, &lTRUBLE, ORD_APPCOL);

// and column Y

      (*(FPROC5)fp)( cNameY, &lNonz, fValY, 
       lIroY, &lTRUBLE, ORD_APPCOL);
   }

// Call GO

   {
      long lLIMGO = 10000000, lISTAT;
      (*(FPROC2)fp)( &lLIMGO, &lISTAT, ORD_GO);
   }

// Get objective and variable values and display them

   {
      long i=1; float fObj, fXPrimal, fYPrimal, fDual;

// Objective

      (*(FPROC3)fp)( &i, &fObj, &fDual, ORD_REPROW);

// X

      (*(FPROC3)fp)( &i, &fXPrimal, &fDual, ORD_REPVAR);

// Y

      i++;
      (*(FPROC3)fp)( &i, &fYPrimal, &fDual, ORD_REPVAR);

      sprintf( cBuff, "Objective = %f X = %f Y = %f", fObj, fXPrimal, fYPrimal);
      MessageBox( NULL, cBuff, cTitle, MB_OK);
   }

// Call LSEXIT to allow LINDO to clean up before exit

   (*(FPROC0) fp)( ORD_LSEXIT);

// Release LINDO library

   FreeLibrary( hDLL);

// All Done

   return( TRUE);
}
//---------------------------------------------------------------------------
