//-----------------------------------------------------------------------------
//
// A small Watcom C program to define and solve the following
// model using the LINDO Dynamic Link Library:
// 
//     Max 20 X + 30 Y
//     S.T.
//         X < 50
//         Y < 60
//         X + 2Y < 120
//     End
//
//-----------------------------------------------------------------------------
//
// Make file:
//
// lndwatc.exe : lndwatc.obj
//  wlink SYS win386 op q name lndwatc fil lndwatc.obj
//  wbind lndwatc -q -n -s C:\WATCOM\binw\win386.ext
// 
// lndwatc.obj : lndwatc.c
//  wcc386 lndwatc.c -i=\watcom\h;\watcom\h\win -zq &
//   -bt=windows -mf
//
//-----------------------------------------------------------------------------
//

// Includes:

#include <windows.h>
#include <stdio.h>
#include <malloc.h>

// The following contains the ordinal definitions of all public routines in the
//  LINDO DLL

#include "lnddllc.h"

//-----------------------------------------------------------------------------

// Main Program:

int PASCAL WinMain( HANDLE hInstance,  HANDLE hPrevInstance, LPSTR lpCmdLine,
		    int nCmdShow )
{

   char buf[256];
   FARPROC fp;
   HANDLE hlib32;
   HINDIR hIndirArgs0, hIndirArgs1, hIndirArgs2, hIndirArgs3, hIndirArgs4,
    hIndirArgs5, hIndirArgs6, hIndirArgs7;

// Load the lindo library

   hlib32 = LoadLibrary( "lindo.dll");

// Get the address of the entry point

   fp = GetProcAddress( hlib32, "Win386LibEntry");

// Define the various type of function calls that may be needed when calling
//  into the LINDO library.  Routines in the LINDO library can have anywhere
//   from 0 to 7 arguments.  LINDO routines ALWAYS expect data to be passed
//    by reference.

// Used for functions with 0 arguments
   hIndirArgs0 = GetIndirectFunctionHandle( fp, INDIR_WORD, INDIR_ENDLIST);
// Used for functions with 1 argument
   hIndirArgs1 = GetIndirectFunctionHandle( fp, INDIR_PTR, INDIR_WORD, 
    INDIR_ENDLIST);
// Used for functions with 2 argument
   hIndirArgs2 = GetIndirectFunctionHandle( fp, INDIR_PTR, INDIR_PTR, 
    INDIR_WORD, INDIR_ENDLIST);
// Used for functions with 3 arguments
   hIndirArgs3 = GetIndirectFunctionHandle( fp, INDIR_PTR, INDIR_PTR, 
    INDIR_PTR, INDIR_WORD, INDIR_ENDLIST);
// Used for functions with 4 arguments
   hIndirArgs4 = GetIndirectFunctionHandle( fp, INDIR_PTR, INDIR_PTR, 
    INDIR_PTR, INDIR_PTR, INDIR_WORD, INDIR_ENDLIST);
// Used for functions with 5 arguments
   hIndirArgs5 = GetIndirectFunctionHandle( fp, INDIR_PTR, INDIR_PTR, 
    INDIR_PTR, INDIR_PTR, INDIR_PTR, INDIR_WORD, INDIR_ENDLIST);
// Used for functions with 6 arguments
   hIndirArgs6 = GetIndirectFunctionHandle( fp, INDIR_PTR, INDIR_PTR, 
    INDIR_PTR, INDIR_PTR, INDIR_PTR, INDIR_PTR, INDIR_WORD, INDIR_ENDLIST);
// Used for functions with 7 arguments
   hIndirArgs7 = GetIndirectFunctionHandle( fp, INDIR_PTR, INDIR_PTR, 
    INDIR_PTR, INDIR_PTR, INDIR_PTR, INDIR_PTR, INDIR_PTR, INDIR_WORD, 
     INDIR_ENDLIST);

// Initialize LINDO by calling ILINDO and INIT.  Note: both functions
//  do not have arguments, so we use hIndirArgs0 in the indirect call

   InvokeIndirectFunction( hIndirArgs0, ORD_ILINDO);
   InvokeIndirectFunction( hIndirArgs0, ORD_INIT);

// Capture any standard output in a file

   {

// First, open a file

      char *cKfname = "LINDO.OUT";
      long lUnit = 60, lFname = 9, lInOrOut = 0, lNotFmt = 0, 
       lLutrmi = 0, lLutrmo = 0;
      InvokeIndirectFunction( hIndirArgs7, &lUnit, &lFname, cKfname, &lInOrOut,
       &lNotFmt, &lLutrmi, &lLutrmo, ORD_LUNOPN);

// Now, tell LINDO to divert all standard output to this unit

      InvokeIndirectFunction( hIndirArgs1, &lUnit, ORD_CAPOUT);
   }

// Put LINDO in TERSE mode

   {
      long lTerse = 0;
      InvokeIndirectFunction( hIndirArgs1, &lTerse, ORD_QUIET);
   }
      
// Call DEFROW to define the rows

   {
      int i;
      long lIdir = -1, lIdrow, lTruble;
      float fRhs = 0., fConstraintRhs[3]={50,60,120};

// Define objective row

      InvokeIndirectFunction( hIndirArgs4, &lIdir, &fRhs, &lIdrow, &lTruble,
       ORD_DEFROW);

// Define constraint rows

      lIdir = 1;
      for ( i = 2; i <= 4; i++)
      {
         InvokeIndirectFunction( hIndirArgs4, &lIdir, &fConstraintRhs[ i - 2], 
          &lIdrow, &lTruble, ORD_DEFROW);
      }
   } 

// Call APPCOL to define our two columns

   {   
      char *cNameX = "X       ";   
      char *cNameY = "Y       ";   
      long lNonz = 3;
      float fValX[] = { 20, 1, 1};
      float fValY[] = { 30, 1, 2};
      long lIroX[] = { 1, 2, 4};
      long lIroY[] = { 1, 3, 4}, lTruble;

// Call APPCOL to define column X

      InvokeIndirectFunction( hIndirArgs5, cNameX, &lNonz, fValX, 
       lIroX, &lTruble, ORD_APPCOL);

// Call APPCOL to define column Y

      InvokeIndirectFunction( hIndirArgs5, cNameY, &lNonz, fValY, 
       lIroY, &lTruble, ORD_APPCOL);
   }

// Call GO to optimize the model

   {
      long lLIMGO = 0, lISTAT;
      InvokeIndirectFunction( hIndirArgs2, &lLIMGO, &lISTAT, ORD_GO);
   }

// Get the objective and variable values and report them in a dialog box

   {
      int i; float fObj, fX, fY, fDual;

// Get the objective value

      i = 1;
      InvokeIndirectFunction( hIndirArgs3, &i, &fObj, &fDual, ORD_REPROW);

// Get the value of X

      InvokeIndirectFunction( hIndirArgs3, &i, &fX, &fDual, ORD_REPVAR);

// Get the value of Y

      i = 2;
      InvokeIndirectFunction( hIndirArgs3, &i, &fY, &fDual, ORD_REPVAR);

// Report values in dialog box

      sprintf( buf, "Objective Value = %g\n\nX = %g\nY = %g", fObj, fX, fY);
      MessageBox( NULL, buf, "Solution", MB_OK | MB_TASKMODAL);
   }

// Shut LINDO down

   InvokeIndirectFunction( hIndirArgs0, ORD_LSEXIT);

// Release the LINDO DLL

   FreeLibrary( hlib32);

// Free our function call handles

   free( hIndirArgs0);
   free( hIndirArgs1);
   free( hIndirArgs2);
   free( hIndirArgs3);
   free( hIndirArgs4);
   free( hIndirArgs5);
   free( hIndirArgs6);
   free( hIndirArgs7);

// All done

   return( 0);
}
