c-----------------------------------------------------------------------------
c
c A small Watcom FORTRAN program to define and solve the following
c model using the LINDO Dynamic Link Library:
c 
c     Max 20 X + 30 Y
c     S.T.
c         X < 50
c         Y < 60
c         X + 2Y < 120
c     End
c
c-----------------------------------------------------------------------------
c
c Make file:
c
c  lndwatf.exe : lndwatf.obj 
c     wlink d all SYS win386 op m op maxe=25 op q NAME lndwatf FIL &
c      lndwatf.obj
c     wbind lndwatf -q -n -s C:\WATCOM\binw\win386.ext
c  
c  lndwatf.obj : lndwatf.for 
c     wfc386 lndwatf.for -incp=\WATCOM\src\fortran\win;\WATCOM\src\fortran&
c       -win -q 
c
c-----------------------------------------------------------------------------
c
*$include winapi.fi

      integer*2 function FWinMain( hInstance, hPrevInstance,
     + lpszCmdLine, nCmdShow)

      implicit none

      integer*2 hInstance
      integer*2 hPrevInstance
      integer*4 lpszCmdLine
      integer*2 nCmdShow

      include 'windows.fi'

      character*128 str

      integer*2 hlib

      integer*4 rc, i, idrow, istat
      integer*4 dll_init, dll_ilindo, dll_quiet, dll_capout
      integer*4 dll_lunopn, dll_defrow, dll_appcol, dll_go
      integer*4 dll_lsexit, dll_reprow, dll_repvar
      integer*4 irowx( 3), irowy( 3)

      logical trouble

      real*4 obj, x, y, dual
      real*4 rhs( 3), valx( 3), valy( 3)

      data rhs /50., 60., 120./
      data valx / 20., 1., 1./
      data valy / 30., 1., 2./
      data irowx /1,2,4/
      data irowy /1,3,4/

c Load the LINDO library

      hlib = LoadLibrary( 'lindo.dll'c )
      if( hlib .lt. 32 ) then
         call MessageBox( NULL, 'Can''t load LINDO'c, ''c, MB_OK)
         stop
      endif

c Initialize LINDO by calling ILINDO and INIT.  Note: ILINDO is the
c sixth routine in the library, and INIT is the first.  For information
c on other routines and their order in the library see file LNDDLLC.H.

      dll_ilindo = GetProcAddress( hlib, 'DLL6'c)
      rc = _Call16( dll_ilindo, ''c)

      dll_init = GetProcAddress( hlib, 'DLL1'c)
      rc = _Call16( dll_init, ''c)

c Capture any standard output in a file

c First, open a file

      dll_lunopn = GetProcAddress( hlib, 'DLL24'c)
      rc = _Call16( dll_lunopn, 'ppppppp'c, loc( 60), loc(9), 
     + loc( 'LINDO.OUT'c), loc( 0), loc( 0), loc( 0), loc(0))

c Now, tell LINDO to send all standard output to this file

      dll_capout = GetProcAddress( hlib, 'DLL4'c)
      rc = _Call16( dll_capout, 'p'c, loc( 60))

c Put LINDO in TERSE mode

      dll_quiet = GetProcAddress( hlib, 'DLL11'c)
      rc = _Call16( dll_quiet, 'p'c, loc( 0))

c Call DEFROW to define objective row

      dll_defrow = GetProcAddress( hlib, 'DLL2'c)
      rc = _Call16( dll_defrow, 'pppp'c, loc( -1), loc( 0.), 
     + loc( idrow), loc( trouble))

c Now, call DEFROW to define constraint rows

      do i = 1, 3
         rc = _Call16( dll_defrow, 'pppp'c, loc( 1), loc( rhs( i)), 
     +    loc( idrow), loc( trouble))
      enddo

c Call APPCOL to define columns

      dll_appcol = GetProcAddress( hlib, 'DLL3'c)

c Define X

      rc = _Call16( dll_appcol, 'ppppp'c, loc( 'X       'c), loc( 3), 
     + loc( valx( 1)), loc( irowx( 1)), loc( trouble))

c Define Y

      rc = _Call16( dll_appcol, 'ppppp'c, loc( 'Y       'c), loc( 3), 
     + loc( valy( 1)), loc( irowy( 1)), loc( trouble))

c Call GO to solve the model

      dll_go = GetProcAddress( hlib, 'DLL8'c)
      rc = _Call16( dll_go, 'pp'c, loc( 1000), loc( istat))

c Get addresses of REPROW and REPVAR

      dll_reprow = GetProcAddress( hlib, 'DLL10'c)
      dll_repvar = GetProcAddress( hlib, 'DLL9'c)

c Get the objective value

      rc = _Call16( dll_reprow, 'ppp'c, loc( 1), loc( obj), loc( dual))
      
c Get the value of x

      rc = _Call16( dll_repvar, 'ppp'c, loc( 1), loc( x), loc( dual))
      
c Get the value of y

      rc = _Call16( dll_repvar, 'ppp'c, loc( 2), loc( y), loc( dual))

c Report values in a dialog box

      write( str, 
     + '(''Objective ='',f8.1,''   X ='',f8.1,''   Y ='',f8.1,1a1)')
     +  obj, x, y, char(0)
      call MessageBox( NULL, str, 
     + 'Watcom FORTRAN LINDO DLL Test'c, MB_OK )

c Shut down LINDO by calling LSEXIT

      dll_lsexit = GetProcAddress( hlib, 'DLL7'c)
      rc = _Call16( dll_lsexit, ''c)

c Release LINDO library

      call FreeLibrary( hlib)

c All Done

      FWinMain = 0

      end
