//---------------------------------------------------------
//
// A small Watcom C program to define and solve the 
// following model using the 32 bit LINDO Dynamic 
// Link Library (LNDDLL32.DLL):
// 
//     Max 20 X + 30 Y
//     S.T.
//         X < 50
//         Y < 60
//         X + 2Y < 120
//     End
//
//---------------------------------------------------------
//
// Make file:
//
// test32c.exe : test32c.obj
//   wlink d all sys nt_win op m libf lnddll32.lib &
//    op maxe=25 op q op m name test32c &
//     file test32c.obj
//   wrc -q -ad test32c.exe
//
// test32c.obj : test32c.c
//   wcc386 test32c.c -i=c:\watcom\h;c:\watcom\nt &
//    -w4 -e25 -zq -5r -bt=nt -mf
//
//---------------------------------------------------------
//

// Includes:

#include <windows.h>
#include <stdio.h>

// lindowat.h contains function defs for all user 
//  routines in the LINDO library

#include "lindowat.h"

//---------------------------------------------------------

// Main Program:

int PASCAL WinMain( HANDLE hInstance,  
                    HANDLE hPrevInstance, 
                    LPSTR lpCmdLine,
                    int nCmdShow )
{

// Initialize LINDO by calling ILINDO and INIT.  

   ILINDO();
   INIT();

// Capture any standard output in a file:

   {

// First, open a file

      char *cKfname = "LINDO.OUT";
      long lUnit = 60, lFname = 9, lInOrOut = 0, 
       lNotFmt = 0, lLutrmi = 0, lLutrmo = 0;
      LUNOPN( &lUnit, &lFname, &cKfname, &lInOrOut,
       &lNotFmt, &lLutrmi, &lLutrmo);

// Now, tell LINDO to divert all standard 
//  output to this unit

      CAPOUT( &lUnit);
   }

// Put LINDO in TERSE mode

   {
      long lTerse = 0;
      QUIET( &lTerse);
   }
      
// Call DEFROW to define the rows

   {
      int i;
      long lIdir = -1, lIdrow, lTruble;
      float fRhs = 0.;
      float fConstraintRhs[3]={50,60,120};

// Define objective row

      DEFROW( &lIdir, &fRhs, &lIdrow, &lTruble);

// Define constraint rows

      lIdir = 1;
      for ( i = 2; i <= 4; i++)
      {
         DEFROW( &lIdir, &fConstraintRhs[ i - 2], 
          &lIdrow, &lTruble);
      }
   } 

// Call APPCOL to define our two columns

   {   
      char *cNameX = "X       ";   
      char *cNameY = "Y       ";   
      long lNonz = 3;
      float fValX[] = { 20, 1, 1};
      float fValY[] = { 30, 1, 2};
      long lIroX[] = { 1, 2, 4};
      long lIroY[] = { 1, 3, 4}, lTruble;

// Call APPCOL to define column X

      APPCOL( &cNameX, &lNonz, fValX, lIroX, &lTruble);

// Call APPCOL to define column Y

      APPCOL( &cNameY, &lNonz, fValY, lIroY, &lTruble);
   }

// Call GO to optimize the model

   {
      long lLIMGO = 0, lISTAT;
      GO( &lLIMGO, &lISTAT);
   }

// Get the objective and variable values and 
//  report them in a dialog box

   {
      char buf[256];
      long i; float fObj, fX, fY, fDual;

// Get the objective value

      i = 1;
      REPROW( &i, &fObj, &fDual);

// Get the value of X

      REPVAR( &i, &fX, &fDual);

// Get the value of Y

      i = 2;
      REPVAR( &i, &fY, &fDual);

// Report values in dialog box

      sprintf( buf, 
       "Objective Value = %g\n\nX = %g\nY = %g", 
        fObj, fX, fY);
      MessageBox( NULL, buf, "Solution", 
       MB_OK | MB_TASKMODAL);
   }

// Shut LINDO down

   LSEXIT();

// All done

   return( 0);
}

